#include "module.h"
#include "main.h"

#include "language.c"
#include "mysql.c"
#include "help.c"
#include "misc.c"

#ifdef USE_SWHOIS
    #include "swhois.c"
#endif
#ifdef USE_SNOMASK
    #include "snomask.c"
#endif
#ifdef USE_HELPOP
    #include "helpop.c"
#endif
#ifdef USE_VHOSTREQ
    #include "vhost.c"
#endif 




int AnopeInit(int argc, char **argv)
{
   Command *c;    


#ifndef USE_MYSQL
    alog("[%s] MYSQL is not enabled. This module requires MYSQL !", MODNAME);
    alog("[%s] Module not loaded.", MODNAME);
    return MOD_STOP;
#endif    
    if (!do_mysql) {
        alog("[%s] MYSQL is not configured. Please edit your services.conf .", MODNAME);
        alog("[%s] Module not loaded.", MODNAME);
        return MOD_STOP;
    }

    /* activate mysql connection and stop if failed */
    /* error messages are logged by mydb_init()     */
    if (mydb_init()) return MOD_STOP; 


    /* load language files */
    mod_lang_init();

    /* now add all the new commands */
    c = createCommand("SET", ns_set, NULL, -1, -1, -1, -1, -1);
        moduleAddCommand(NICKSERV, c, MOD_HEAD);
    c = createCommand("IDENTIFY", do_on_identify, NULL, -1, -1, -1, -1, -1);
        moduleAddCommand(NICKSERV, c, MOD_TAIL);
    c = createCommand("ID", do_on_identify, NULL, -1, -1, -1, -1, -1);
        moduleAddCommand(NICKSERV, c, MOD_TAIL);
    c = createCommand("UPDATE", do_on_identify, NULL, -1, -1, -1, -1, -1);
        moduleAddCommand(NICKSERV, c, MOD_TAIL);
    c = createCommand("LOGIN", do_on_identify, NULL, -1, -1, -1, -1, -1);
        moduleAddCommand(NICKSERV, c, MOD_UNIQUE);
                    /* the LOGIN command is only added for compatibility 
                       with the old epona modification made by Scifi */
    c = createCommand("HELP", ns_help_set, NULL, -1, -1, -1, -1, -1);
        moduleAddCommand(NICKSERV, c, MOD_TAIL);

    c = createCommand("SET ALOGIN", NULL,  NULL, -1, -1, -1, -1, -1);
        moduleAddCommand(NICKSERV, c, MOD_TAIL);
        moduleAddHelp(c,ns_help_set_alogin);
#ifdef USE_NS_INFO    /* replace the "/nickserv info" command */
    c = createCommand("INFO", do_info2, NULL, -1, -1, -1, -1, -1);
        moduleAddCommand(NICKSERV, c, MOD_HEAD);
#endif /* USE_NS_INFO */
#ifdef USE_REALNAME
    if (ircd->chgreal) {
        c = createCommand("SET REALNAME",  NULL,  NULL, -1, -1, -1, -1, -1);
        moduleAddCommand(NICKSERV, c, MOD_TAIL);
        moduleAddHelp(c,ns_help_set_realname);
    }
#endif /* USE_REALNAME */
#ifdef USE_SWHOIS
        c = createCommand("SWHOIS", os_swhois, is_services_oper, -1, -1, -1, -1, -1);
            moduleAddCommand(OPERSERV, c, MOD_HEAD);
            moduleAddHelp(c,os_help_swhois);
#endif /* USE_SWHOIS */
#ifdef USE_SNOMASK
        c = createCommand("SNOMASK", os_snomask, is_services_oper, -1, -1, -1, -1, -1);
            moduleAddCommand(OPERSERV, c, MOD_HEAD);
            moduleAddHelp(c, os_help_snomask);
#endif /* USE_SNOMASK */
#ifdef USE_HELPOP
        c = createCommand("HELPOP", os_helpop, is_services_admin, -1, -1, -1, -1, -1);
            moduleAddCommand(OPERSERV, c, MOD_HEAD);
            moduleAddHelp(c, os_help_helpop);
#endif /* USE_HELPOP */
#ifdef USE_VHOSTREQ
    if (ircd->vhost && ircd->vident) {
        c = createCommand("VHOST", os_vhost, is_services_oper, -1, -1, -1, -1, -1);
            moduleAddCommand(OPERSERV, c, MOD_HEAD);
            moduleAddHelp(c, os_help_vhost);
        c = createCommand("VHOST PENDING", NULL, NULL, -1, -1, -1, -1, -1);
            moduleAddCommand(OPERSERV, c, MOD_TAIL);
            moduleAddHelp(c, os_help_vhost_pending);
        c = createCommand("SET VHOST", NULL,  NULL, -1, -1, -1, -1, -1);
            moduleAddCommand(NICKSERV, c, MOD_TAIL);
            moduleAddHelp(c,ns_help_set_vhost);
    }
#endif /* USE_VHOSTREQ */
#ifdef USE_AJOIN
    c = createCommand("SET AJOIN", NULL,  NULL, -1, -1, -1, -1, -1);
        moduleAddCommand(NICKSERV, c, MOD_TAIL);
        moduleAddHelp(c,ns_help_set_ajoin);
#endif /* USE_AJOIN */

    
    /* add the new commands to operserv help */
    moduleSetOperHelp(help_operserv);

    /* start database cleanup function */
    /* purgedb(0,NULL); */
    moduleAddAuthor(AUTHOR);
    moduleAddVersion(VERSION);
    return MOD_CONT;
}

void AnopeFini(void)
{
    /* closing mysql connection */
    mysql_close(mymysql);
    /* unloading language files */
    mod_lang_unload();
    /* module unloading */
}


