#include "misc.h"

/***************************************************************************************************/

/* identify */

int do_on_identify(User *u) {
    extern MYSQL *mymysql;
    char *temp = moduleGetLastBuffer();
    char *buf;
    int alogin_     = 0;
    int silent_     = 0;
    int nologin_    = 0;
    int novhost_    = 0;
    int nojoin_     = 0;
    int noswhois_   = 0;
    int nohelpop_   = 0;
    int nosnomask_  = 0;
    int norealname_ = 0;

    /* stop if nick is not identified */
    if (!nick_identified(u)) { return MOD_CONT; }

    /* get options from commandline */
    if (temp) {  /* temp can be NULL if triggered by "/ns update" */
        if (stristr(temp,"nologin"))   { nologin_    = 1; }
        if (stristr(temp,"novhost"))   { novhost_    = 1; }
        if (stristr(temp,"nojoin"))    { nojoin_     = 1; }
        if (stristr(temp,"noswhois"))  { noswhois_   = 1; }
        if (stristr(temp,"nohelpop"))  { nohelpop_   = 1; }
        if (stristr(temp,"nosnomask")) { nosnomask_  = 1; }
        if (stristr(temp,"norealname")){ norealname_ = 1; }
    }

    qbuf1 = quote(u->nick); 
             /*        0          1        2      3      4       5        6          7            8 */
    do_query("SELECT alogin, vhoststatus, vhost, ajoin, swhois, helpop, snomasks, realname, vhostreqstatus "
                                "FROM "DBNAME" WHERE nick = '%s';", qbuf1);
    free(qbuf1);
    myres = mysql_store_result(mymysql);
    if (mysql_num_rows(myres) > 0) {
        myrow=mysql_fetch_row(myres);
        if (!myrow[0]) {
                anope_cmd_global(s_NickServ, "ERROR! [%s]: field \"alogin\" for "
                            "user \"%s\" contains an invalid value! (NULL)", MODNAME, u->nick);
                mysql_free_result(myres);
                return MOD_STOP;
        }
        if (stricmp(myrow[0],"1")==0) { alogin_ = 1; }
        if (stricmp(myrow[0],"2")==0) { alogin_ = 1; silent_ = 1; }
        if ((alogin_ || silent_) && !nologin_) {                /*  alogin on ? */
#ifdef USE_VHOSTREQ
            if (ircd->vhost) {
                if (stricmp(myrow[1],"1")==0) {
                    if (!novhost_) {
                        if (!silent_) { noticeLang(s_NickServ, u, VHOST_NOTICE_VHOST); }
                        set_vhost(u, myrow[2]);
                    }
                }
            }
#endif
#ifdef USE_SWHOIS
            if (myrow[4] && strlen(myrow[4]) > 0) {         /* swhois */
                if (!noswhois_) {
                        if (!silent_) { noticeLang(s_NickServ, u, SWHOIS_NOTICE_SWHOIS); }
                        anope_cmd_swhois(s_NickServ, u->nick, myrow[4]);
                }
            }
#endif
#ifdef USE_HELPOP
            if (myrow[5] && strlen(myrow[5]) > 0) {         /* user modes */
                if (!nohelpop_) {
                        if (!silent_) { noticeLang(s_NickServ, u, HELPOP_NOTICE_HELPOP);  }
                        anope_cmd_svsmode(u, 1 , &myrow[5]);   
                }
            }
#endif
#ifdef USE_SNOMASK
                if (myrow[6] && strlen(myrow[6]) > 0) {         /* snomask */
                    if (!nosnomask_) {
                        if (!silent_) { noticeLang(s_NickServ, u, SNOMASK_NOTICE_SNOMASK); }
                        send_cmd(s_NickServ, "SVSSNO %s %s", u->nick, myrow[6]);
                    }
                }
#endif
#ifdef USE_REALNAME
            if (ircd->chgreal) {
                if (myrow[7] && strlen(myrow[7]) > 0) { /* realname */
                    if (!norealname_) {
                        if (!silent_) { noticeLang(s_NickServ, u, REALNAME_NOTICE_SET, myrow[7]); }
                        send_cmd(s_NickServ, "CHGNAME %s %s",  u->nick, myrow[7]);
                        change_user_realname(u, myrow[7]); /* update internal struct */
                    }
                }
            }
#endif
#ifdef USE_AJOIN
            if (myrow[3] &&  strlen(myrow[3]) > 0) {        /* ajoin */
                if (!nojoin_) {
                    if (!silent_) { noticeLang(s_NickServ, u, AJOIN_NOTICE_AJOIN, myrow[3]); }
                            /* anope_cmd_svsjoin(char *source, char *nick, char *chan, char *param) */
                    anope_cmd_svsjoin(s_NickServ, u->nick, myrow[3], NULL); 
                }
            }
#endif
#ifdef USE_VHOSTREQ
            if (ircd->vhost) {
                if (myrow[8] && strlen(myrow[8]) > 0) { /* information notice about vhost requesting */
                        buf = sstrdup(myrow[8]);  /* for some strange reason a "stricmp(myrow[8],"2")"         */
                                                  /* sometimes causes a segfault .... we use buf to prevent it  */
                        if (stricmp(buf,"2")==0) {
                                noticeLang(s_NickServ, u, VHOST_NOTICE_ACCEPT);
                                add_to_db(u->na, "vhostreqstatus", "0");
                        }
                        if (stricmp(buf,"3")==0) {
                                noticeLang(s_NickServ, u, VHOST_NOTICE_REJECT);
                                add_to_db(u->na, "vhostreqstatus", "0");
                        }
                        if (stricmp(buf,"4")==0) {
                                noticeLang(s_NickServ, u, VHOST_NOTICE_NEW_VHOST);
                                add_to_db(u->na, "vhostreqstatus", "0");
                        }
                        if (stricmp(buf,"5")==0) {
                                noticeLang(s_NickServ, u, VHOST_NOTICE_DEL_VHOST);
                                add_to_db(u->na, "vhostreqstatus", "0");
                        }
                        free(buf);
                }
            }
#endif
        }
    }   
    mysql_free_result(myres); /* release unused memory */
    return MOD_CONT;
}

/***************************************************************************************************/

/* /nickserv set ... */


int ns_set(User *u) {
    char *temp = moduleGetLastBuffer();
    char *cmd  = myStrGetToken(temp,' ',0);
    char *option1 = myStrGetToken(temp,' ',1);

    if (!cmd) { goto MODCONT; }

    if (!nick_identified(u)) {
        notice_lang(s_NickServ, u, NICK_IDENTIFY_REQUIRED, s_NickServ);
        goto MODSTOP;
    }

    if (stricmp(cmd, "alogin")==0) {
        if (option1) {
            if (stricmp(option1,"on")==0) {
                get_from_db(u->nick, "alogin");
                if (stricmp(myresult,"1")==0) {
                    noticeLang(s_NickServ, u, ALOGIN_NOTICE_IS_ON);
                    goto MODSTOP;
                }
                add_to_db(u->na, "alogin", "1");
                noticeLang(s_NickServ, u, ALOGIN_NOTICE_IS_NOW_ON);
                goto MODSTOP;
            }
            if (stricmp(option1,"off")==0) {
                get_from_db(u->nick, "alogin");
                if (stricmp(myresult,"0")==0) {
                    noticeLang(s_NickServ, u, ALOGIN_NOTICE_IS_OFF);
                    goto MODSTOP;
                }
                add_to_db(u->na, "alogin", "0");
                noticeLang(s_NickServ, u, ALOGIN_NOTICE_IS_NOW_OFF);
                goto MODSTOP;
            }
            if (stricmp(option1,"silent")==0) {
                get_from_db(u->nick, "alogin");
                if (stricmp(myresult,"2")==0) {
                    noticeLang(s_NickServ, u, ALOGIN_NOTICE_IS_SIL);
                    goto MODSTOP;
                }
                add_to_db(u->na, "alogin", "2");
                noticeLang(s_NickServ, u, ALOGIN_NOTICE_IS_NOW_SIL);
                goto MODSTOP;
            }

        }
        noticeLang(s_NickServ, u, ALOGIN_SYNTAX);
        goto MODSTOP;
    }
#ifdef USE_AJOIN
/* todo: replace with new ajoin function */
        if (stricmp(cmd, "ajoin")==0) {
            if (!option1) {
                noticeLang(s_NickServ, u, AJOIN_SYNTAX);
                goto MODSTOP;
            }
            if (stricmp(option1, "list")==0) {
                get_from_db(u->nick,"ajoin");
                if (myresult) {
                    noticeLang(s_NickServ, u, AJOIN_NOTICE_AJOINLIST, myresult); /* your ajoin chans are ... */
                } else {
                    noticeLang(s_NickServ, u, AJOIN_NOTICE_EMPTY_LIST); /* you have no ajoin chans set */
                }
                goto MODSTOP;
            }
            if (stricmp(option1, "off")==0) {
                add_to_db(u->na, "ajoin", "");
                noticeLang(s_NickServ, u, AJOIN_NOTICE_EMPTY_LIST_NOW); /* your ajoin list is now empty */
                goto MODSTOP;
            }
            if (option1) free(option1);
            option1 = myStrGetTokenRemainder(temp,' ',1); /* get full line including the channel passwords */
            if (strlen(option1) > 511) {
                noticeLang(s_NickServ, u, AJOIN_NOTICE_MAX_AJOIN); /* ajoin list too long */
                goto MODSTOP;
            }
            add_to_db(u->na, "ajoin", option1);
            noticeLang(s_NickServ, u, AJOIN_NOTICE_AJOINLIST_NEW, option1);   /* your ajoin chans are now: ... */
            goto MODSTOP;
        }
#endif
#ifdef USE_REALNAME
    if (ircd->chgreal) {
        if (stricmp(cmd, "realname")==0) {
            if (option1) {
                free(option1);
                option1 = myStrGetTokenRemainder(temp,' ',1);
                add_to_db(u->na, "realname", option1);
                send_cmd(s_NickServ, "CHGNAME %s %s",  u->nick, option1);
                change_user_realname(u, option1); /* update internal struct */
                noticeLang(s_NickServ, u, REALNAME_NOTICE_NEW, option1);
                goto MODSTOP;
            }
            noticeLang(s_NickServ, u, REALNAME_SYNTAX);
            goto MODSTOP;
        }
    }
#endif
#ifdef USE_VHOSTREQ
    if (ircd->vhost) {
        if (stricmp(cmd, "vhost") == 0) {
            if (!option1) {
                noticeLang(s_NickServ, u, VHOST_NS_SYNTAX);
                goto MODSTOP;
            }
            if (stricmp(option1,"on")==0) {
                get_from_db(u->nick, "vhost");
                if (!myresult || (strlen(myresult) < 2)) {
                    noticeLang(s_NickServ, u, VHOST_NOTICE_NO_VHOST);
                    goto MODSTOP;
                }
                get_from_db(u->nick, "vhoststatus");
                if (stricmp(myresult,"1")==0) {
                    noticeLang(s_NickServ, u, VHOST_NOTICE_IS_ON);
                    goto MODSTOP;
                }
                add_to_db(u->na, "vhoststatus", "1");
                noticeLang(s_NickServ, u, VHOST_NOTICE_IS_NOW_ON);
                goto MODSTOP;
            }
            if (stricmp(option1,"off")==0) {
                get_from_db(u->nick, "vhost");
                if (!myresult || (strlen(myresult) < 2)) {
                    noticeLang(s_NickServ, u, VHOST_NOTICE_NO_VHOST);
                    goto MODSTOP;
                }
                get_from_db(u->nick, "vhoststatus");
                if (stricmp(myresult,"0")==0) {
                    noticeLang(s_NickServ, u, VHOST_NOTICE_IS_OFF);
                    goto MODSTOP;
                }
                add_to_db(u->na, "vhoststatus", "0");
                noticeLang(s_NickServ, u, VHOST_NOTICE_IS_NOW_OFF);
                goto MODSTOP;
            }
            /* grouping */
            if (stricmp(option1,"group")==0) {
                get_from_db(u->na->nc->display, "vhost");
                if (!myresult) {
                    notice_user(s_NickServ, u, "No vHost to set");
                    goto MODSTOP;
                }
                if (strlen(myresult) < 3) {
                    notice_user(s_NickServ, u, "No vHost to set");
                    goto MODSTOP;
                }
                noticeLang(s_NickServ, u, VHOST_NOTICE_NEW_VHOST_SELFSET, myresult );
                qbuf3 = quote(myresult);
                add_to_db(u->na, "vhost", qbuf3);
                free(qbuf3);
                add_to_db(u->na, "vhoststatus", "1"); /* active */
                add_to_db(u->na, "vhostreqstatus", "0"); 
                add_to_db(u->na, "v_setter", "NS GROUP"); /* vhost is set by the group command */
                goto MODSTOP;
            }
            if (!is_valid_vhost(u, option1)) {
                goto MODSTOP;
            }
            add_to_db(u->na, "vhostreqstatus", "1");
            add_to_db(u->na, "vhostreq", option1);
            noticeLang(s_NickServ, u, VHOST_NOTICE_REQUEST);
            anope_cmd_global(s_NickServ, "\2%s\2 has requested a new vhost (\2%s\2). "
                "do \2/operserv vhost pending list\2 to view the pending vhosts\002", u->nick, option1);
            goto MODSTOP;
        }
    }
#endif /* USE_VHOSTREQ */
    goto MODCONT;

    MODSTOP:
        if (cmd) free(cmd);
        if (option1) free(option1);
        return MOD_STOP;

    MODCONT:
        if (cmd) free(cmd);
        if (option1) free(option1);
        return MOD_CONT;

}

/* end of nickserv set */

/*****************************************************************************************/
/* this is a copy of do_info() from src/core/ns_info.c with some modifications           */
/* it replaces the /nickserv info command and displays additional informations           */
/*****************************************************************************************/

int do_info2(User * u)
{

/* Show hidden info to nick owners and sadmins when the "ALL" parameter is
 * supplied. If a nick is online, the "Last seen address" changes to "Is
 * online from".
 * Syntax: INFO <nick> {ALL}
 * -TheShadow (13 Mar 1999)
 */

    char *nick = strtok(NULL, " ");
    char *param = strtok(NULL, " ");

    NickAlias *na;
    NickRequest *nr = NULL;
        /* Being an oper is enough from now on -GD */
    int is_servadmin = is_services_oper(u);

    if (!nick) {
        syntax_error(s_NickServ, u, "INFO", NICK_INFO_SYNTAX);
    } else if (!(na = findnick(nick))) {
        if ((nr = findrequestnick(nick))) {
            notice_lang(s_NickServ, u, NICK_IS_PREREG);
            if (param && stricmp(param, "ALL") == 0 && is_servadmin) {
                notice_lang(s_NickServ, u, NICK_INFO_EMAIL, nr->email);
            } else {
                if (is_servadmin) {
                    notice_lang(s_NickServ, u, NICK_INFO_FOR_MORE,
                                s_NickServ, nr->nick);
                }
            }
        } else if (nickIsServices(nick, 1)) {
            notice_lang(s_NickServ, u, NICK_X_IS_SERVICES, nick);
        } else {
            notice_lang(s_NickServ, u, NICK_X_NOT_REGISTERED, nick);
        }
    } else if (na->status & NS_VERBOTEN) {
        if (is_oper(u) && na->last_usermask)
            notice_lang(s_NickServ, u, NICK_X_FORBIDDEN_OPER, nick,
                        na->last_usermask,
                        (na->last_realname ? na->
                         last_realname : getstring(u->na, NO_REASON)));
        else
            notice_lang(s_NickServ, u, NICK_X_FORBIDDEN, nick);
    } else {
        struct tm *tm;
        char buf[BUFSIZE], *end;
        const char *commastr = getstring(u->na, COMMA_SPACE);
        int need_comma = 0;
        int nick_online = 0;
        int show_hidden = 0;
        time_t expt;

        /* Is the real owner of the nick we're looking up online? -TheShadow */
        if (na->status & (NS_RECOGNIZED | NS_IDENTIFIED))
            nick_online = 1;

        /* Only show hidden fields to owner and sadmins and only when the ALL
         * parameter is used. -TheShadow */
        if (param && stricmp(param, "ALL") == 0 && u->na
            && ((nick_identified(u) && (na->nc == u->na->nc))
                || is_servadmin))
            show_hidden = 1;

        notice_lang(s_NickServ, u, NICK_INFO_REALNAME, na->nick,
                    na->last_realname);

        if ((nick_identified(u) && (na->nc == u->na->nc)) || is_servadmin) {


            if (nick_is_services_root(na->nc))
                notice_lang(s_NickServ, u, NICK_INFO_SERVICES_ROOT,
                            na->nick);
            else if (nick_is_services_admin(na->nc))
                notice_lang(s_NickServ, u, NICK_INFO_SERVICES_ADMIN,
                            na->nick);
            else if (nick_is_services_oper(na->nc))
                notice_lang(s_NickServ, u, NICK_INFO_SERVICES_OPER,
                            na->nick);

        } else {

            if (nick_is_services_root(na->nc)
                && !(na->nc->flags & NI_HIDE_STATUS))
                notice_lang(s_NickServ, u, NICK_INFO_SERVICES_ROOT,
                            na->nick);
            else if (nick_is_services_admin(na->nc)
                     && !(na->nc->flags & NI_HIDE_STATUS))
                notice_lang(s_NickServ, u, NICK_INFO_SERVICES_ADMIN,
                            na->nick);
            else if (nick_is_services_oper(na->nc)
                     && !(na->nc->flags & NI_HIDE_STATUS))
                notice_lang(s_NickServ, u, NICK_INFO_SERVICES_OPER,
                            na->nick);

        }

        if (nick_online) {
            if (show_hidden || !(na->nc->flags & NI_HIDE_MASK))
                notice_lang(s_NickServ, u, NICK_INFO_ADDRESS_ONLINE,
                            na->last_usermask);
            else
                notice_lang(s_NickServ, u, NICK_INFO_ADDRESS_ONLINE_NOHOST,
                            na->nick);
        } else {
            if (show_hidden || !(na->nc->flags & NI_HIDE_MASK))
                notice_lang(s_NickServ, u, NICK_INFO_ADDRESS,
                            na->last_usermask);
        }

        tm = localtime(&na->time_registered);
        strftime_lang(buf, sizeof(buf), u, STRFTIME_DATE_TIME_FORMAT, tm);
        notice_lang(s_NickServ, u, NICK_INFO_TIME_REGGED, buf);

        if (!nick_online) {
            tm = localtime(&na->last_seen);
            strftime_lang(buf, sizeof(buf), u, STRFTIME_DATE_TIME_FORMAT,
                          tm);
            notice_lang(s_NickServ, u, NICK_INFO_LAST_SEEN, buf);
        }

        if (na->last_quit
            && (show_hidden || !(na->nc->flags & NI_HIDE_QUIT)))
            notice_lang(s_NickServ, u, NICK_INFO_LAST_QUIT, na->last_quit);

        if (na->nc->url)
            notice_lang(s_NickServ, u, NICK_INFO_URL, na->nc->url);
        if (na->nc->email
            && (show_hidden || !(na->nc->flags & NI_HIDE_EMAIL)))
            notice_lang(s_NickServ, u, NICK_INFO_EMAIL, na->nc->email);
        if (na->nc->icq)
            notice_lang(s_NickServ, u, NICK_INFO_ICQ, na->nc->icq);

        if (show_hidden) {
/**** MODIFIED ******/
            qbuf1 = quote(nick);
                        /*      0       1      2       3       4          5    */
            do_query("SELECT  vhost, ajoin, swhois, helpop, snomasks, realname "
                    "FROM `" DBNAME "` WHERE `nick` = '%s';", qbuf1);
            free(qbuf1);
            myres = mysql_store_result(mymysql);
            if (mysql_num_rows(myres) > 0) {
                myrow = mysql_fetch_row(myres);
#ifdef USE_VHOSTREQ
                if (myrow[0] && strlen(myrow[0]) > 0)   { notice_user(s_NickServ, u,"            vhost: %s", myrow[0]); }
#endif
#ifdef USE_AJOIN
                /* todo: replace this by a function to return the string of all ajoin-channels (without password) */ 
                if (myrow[1] && strlen(myrow[1]) > 0)   { notice_user(s_NickServ, u,"      ajoin chans: %s", myrow[1]); }
#endif
#ifdef USE_SWHOIS
                if (myrow[2] && strlen(myrow[2]) > 0)   { notice_user(s_NickServ, u,"           swhois: %s", myrow[2]); }
#endif
#ifdef USE_HELPOP
                if (myrow[3] && strlen(myrow[3]) > 0)   { notice_user(s_NickServ, u,"     helpop modes: %s", myrow[3]); }
#endif
#ifdef USE_SNOMASK
                if (myrow[4] && strlen(myrow[4]) > 0)   { notice_user(s_NickServ, u,"          snomask: %s", myrow[4]); }
#endif
            }
            mysql_free_result(myres);  /* free unused memory */

/**** /MODIFIED *********/


            if (na->nc->greet)
                notice_lang(s_NickServ, u, NICK_INFO_GREET, na->nc->greet);

            *buf = 0;
            end = buf;

            if (na->nc->flags & NI_KILLPROTECT) {
                end += snprintf(end, sizeof(buf) - (end - buf), "%s",
                                getstring(u->na, NICK_INFO_OPT_KILL));
                need_comma = 1;
            }
            if (na->nc->flags & NI_SECURE) {
                end += snprintf(end, sizeof(buf) - (end - buf), "%s%s",
                                need_comma ? commastr : "",
                                getstring(u->na, NICK_INFO_OPT_SECURE));
                need_comma = 1;
            }
            if (na->nc->flags & NI_PRIVATE) {
                end += snprintf(end, sizeof(buf) - (end - buf), "%s%s",
                                need_comma ? commastr : "",
                                getstring(u->na, NICK_INFO_OPT_PRIVATE));
                need_comma = 1;
            }
            if (na->nc->flags & NI_MSG) {
                end += snprintf(end, sizeof(buf) - (end - buf), "%s%s",
                                need_comma ? commastr : "",
                                getstring(u->na, NICK_INFO_OPT_MSG));
                need_comma = 1;
            }
            if (!(na->nc->flags & NI_AUTOOP)) {
                end += snprintf(end, sizeof(buf) - (end - buf), "%s%s",
                                need_comma ? commastr : "",
                                getstring(u->na, NICK_INFO_OPT_AUTOOP));
                need_comma = 1;
            }

            notice_lang(s_NickServ, u, NICK_INFO_OPTIONS,
                        *buf ? buf : getstring(u->na, NICK_INFO_OPT_NONE));

            if (na->nc->flags & NI_SUSPENDED) {
                if (na->last_quit) {
                    notice_lang(s_NickServ, u, NICK_INFO_SUSPENDED,
                                na->last_quit);
                } else {
                    notice_lang(s_NickServ, u,
                                NICK_INFO_SUSPENDED_NO_REASON);
                }
            }

            if (na->status & NS_NO_EXPIRE) {
                notice_lang(s_NickServ, u, NICK_INFO_NO_EXPIRE);
            } else {
                if (is_services_admin(u)) {
                    expt = na->last_seen + NSExpire;
                    tm = localtime(&expt);
                    strftime_lang(buf, sizeof(buf), na->u,
                                  STRFTIME_DATE_TIME_FORMAT, tm);
                    notice_lang(s_NickServ, u, NICK_INFO_EXPIRE, buf);
                }
            }
        }

        if (!show_hidden
            && ((u->na && (na->nc == u->na->nc) && nick_identified(u))
                || is_servadmin))
            notice_lang(s_NickServ, u, NICK_INFO_FOR_MORE, s_NickServ,
                        na->nick);
    }
    return MOD_STOP;  /* dont continue with the core function do_info */
}
